function [OK, nConvert] = CMTM10_SimulinkModelUpdate()
% Updates CM4SL/TM4SL Simulink models of Type CM9 to Type CM10

clc;

OK = -1;
nConvert = 0;


fprintf('=====================================\n');
fprintf('Starting CM4SL/TM4SL models update script\n\n');
fprintf('Executing cmenv.m to ensure the necessary paths are included\n\n');
cmenv;

if(~isempty(bdroot))
    if strcmp(get_param(bdroot,'Dirty'),'on')
        fprintf('Aborting update script due to unsaved simulink models.\n');
        fprintf('Ensure all Simulink models are saved and closed before executing script\n');
        fprintf('=====================================\n');
        return
    end
end

fprintf('\n');
fprintf('Closing all open CM4SL/TM4SL models\n\n');
bdclose('all');

% scan all simulink models with extension .mdl and .slx
FileList = [dir('*.slx')' dir('*.mdl')'];
for i=1:numel(FileList)
    Mdl_FileName = FileList(i).name;
    [Mdl_FilePath,Mdl_BaseName,Mdl_FileExt] = fileparts(FileList(i).name);
    open_system(Mdl_FileName);
    % Updating CM4SL/TM4SL models without Suspension Forces block and if not updated before
    if ((numel(find_system(bdroot,'Name','CarMaker'))==1 || numel(find_system(bdroot,'Name','TruckMaker'))==1) && numel(find_system(bdroot,'Name','Suspension Forces'))==0)
        if (numel(find_system(bdroot,'Name','CarMaker'))==1)
            XM = 'CM';
            XMaker = 'CarMaker';
        elseif (numel(find_system(bdroot,'Name','TruckMaker'))==1)
            XM = 'TM';
            XMaker = 'TruckMaker';
        end
        % Saving a backup of the simulation model if not yet existing
        if (isempty(regexp(Mdl_FileName,['\w+\_' XM '9_backup\.\w+'])) && exist([Mdl_BaseName '_' XM '9_backup' Mdl_FileExt]) ~= 4)
            [status,message,messageId] = copyfile (Mdl_FileName, [Mdl_BaseName '_' XM '9_backup' Mdl_FileExt]);
            fprintf('Backuped: %s -> %s\n', Mdl_FileName, [Mdl_BaseName '_' XM '9_backup' Mdl_FileExt]);
            % TODO - overrride Backup?
        end
        % Updating the simulation model if the model is not the backup file
        if (isempty(regexp(Mdl_FileName,['\w+\_' XM '9_backup\.\w+'])))
            sim_system=bdroot;
            % Updating the annotation text containing CarMaker version (valid for TM4SL models as well)
            annotationObjs = find_system(bdroot,'FindAll','on','Type','annotation');
            for j=1:numel(annotationObjs)
                if regexp(get_param(annotationObjs(j),'PlainText'),'CarMaker 9.*')
                    annObj = get_param(annotationObjs(j),'Object');
                    annObj.Text = 'CarMaker 10.0';
                end
            end

            % Updating the simulation model (including the Suspension Forces block)
            susp_system=load_system(['SuspForces_' XM '10_Blockset.mdl']);
            susp_block=add_block(['SuspForces_' XM '10_Blockset/Suspension Forces'],[sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces/Suspension Forces'],'MakeNameUnique','on');
            susp_block_name=get_param(susp_block,'Name');
            close_system(susp_system)
            delete_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],'Vehicle Forces/1','Sync_Out/1');
            set_param([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces/' susp_block_name],'Position',[135 632 260 788])
            add_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],'Vehicle Forces/1',[susp_block_name '/1'],'autorouting','on');
            add_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],[susp_block_name '/1'],'Sync_Out/1','autorouting','on');
            add_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],'lSpring/1',[susp_block_name '/2'],'autorouting','on');
            add_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],'vDamp/1',[susp_block_name '/3'],'autorouting','on');
            add_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],'lStabi/1',[susp_block_name '/4'],'autorouting','on');
            add_line([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces'],'lBuf/1',[susp_block_name '/5'],'autorouting','on');
            set_param([sim_system '/' XMaker '/IPG Vehicle/CarAndTrailer/Forces/' 'Sync_Out'],'Position',[365 58 395 72]);

            % Saving the updated simulation model
            save_system(sim_system);

            fprintf('Updated: %s\n', Mdl_FileName);
            nConvert = nConvert + 1;
        else
            % Skipping backup models
            fprintf('Skipped: %s - Backup file!\n', Mdl_FileName);
        end
    else
        % Skipping models (already updated or not CM4SL/TM4SL models)
        fprintf('Skipped: %s - already updated or no CM4SL/TM4SL model\n', Mdl_FileName);
    end
    close_system;
end

fprintf('\n');
fprintf('Finishing CM4SL/TM4SL models update script\n');
fprintf('%d file updated\n', nConvert);
fprintf('=====================================\n');

%TODO - error handling 
OK = 1;
end

